import numpy as np
import scipy.io as sio
import matplotlib.pyplot as plt


def window_filter(raw_data, weights):
    """
    Filter raw data using window of weights.

    :rtype: np.array
    :type raw_data: np.array
    :type weights: np.array
    """
    # Make sure that element weights are a NumPy vector
    element_weights = np.array(weights, dtype=float).flatten()
    # Normalize them
    element_weights /= np.sum(element_weights)
    # And determine the window size
    window_size = len(element_weights)

    # Now initialise the output by explicitly copying the input as float
    # Note: Simple assignment would not work as we would end with a reference to the same data
    # Note: np.copy() would work but the result would be of the same dtype as the original array,
    # i.e. uint8, and that would cause the np.sum() result to be cased back to this type
    filt_data = np.array(raw_data, dtype=float)

    # Probably not very efficient in terms of array access
    for index in range(window_size, len(raw_data)):
        data_window = raw_data[(index - window_size):index]
        filt_data[index] = np.sum(data_window * element_weights)

    return filt_data


matfile_contents = sio.loadmat('data.mat')
# Note: data type is uint8, not float, so every operation will result in uint8 value
data_in = matfile_contents['data']
raw_data = data_in.flatten()

w05u = window_filter(raw_data, [1, 1, 1, 1, 1])  # All elements are equally important
w05w = window_filter(raw_data, [1, 1, 1, 1, 10])  # Last element is much more important
w10u = window_filter(raw_data, np.ones([1, 10]))
w20u = window_filter(raw_data, np.ones([1, 20]))

t = range(len(raw_data))

plt.figure(1)
plt.plot(t, raw_data, t, w05u, t, w10u, t, w20u)
plt.show()